@@ -12,19 +12,22 @@
 import contextlib
 
 import docker
-from six.moves.urllib import parse
 from tempest.common import credentials_factory as common_creds
 from tempest import config
 from tempest.lib.common import api_version_utils
 from tempest.lib.common import rest_client
 from tempest.lib.services.image.v2 import images_client
+from tempest.lib.services.network import floating_ips_client
 from tempest.lib.services.network import networks_client
 from tempest.lib.services.network import ports_client
+from tempest.lib.services.network import routers_client
+from tempest.lib.services.network import security_group_rules_client
 from tempest.lib.services.network import security_groups_client
 from tempest.lib.services.network import subnetpools_client
 from tempest.lib.services.network import subnets_client
 from tempest.lib.services.volume.v3 import volumes_client
 from tempest import manager
+from urllib import parse
 
 from zun_tempest_plugin.tests.tempest.api.models import capsule_model
 from zun_tempest_plugin.tests.tempest.api.models import container_model
@@ -80,6 +83,10 @@ class Manager(manager.Manager):
         self.sgs_client = security_groups_client.SecurityGroupsClient(
             self.auth_provider, 'network', CONF.identity.region,
             disable_ssl_certificate_validation=True)
+        self.sg_rules_client = \
+            security_group_rules_client.SecurityGroupRulesClient(
+                self.auth_provider, 'network', CONF.identity.region,
+                disable_ssl_certificate_validation=True)
         self.vol_client = volumes_client.VolumesClient(
             self.auth_provider, 'volumev3', CONF.identity.region,
             disable_ssl_certificate_validation=True)
@@ -93,6 +100,12 @@ class Manager(manager.Manager):
         self.subnetpools_client = subnetpools_client.SubnetpoolsClient(
             self.auth_provider, 'network', CONF.identity.region,
             disable_ssl_certificate_validation=True)
+        self.fip_client = floating_ips_client.FloatingIPsClient(
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
+        self.routers_client = routers_client.RoutersClient(
+            self.auth_provider, 'network', CONF.identity.region,
+            disable_ssl_certificate_validation=True)
 
 
 class ZunClient(rest_client.RestClient):
@@ -118,8 +131,8 @@ class ZunClient(rest_client.RestClient):
     def request(self, *args, **kwargs):
         resp, resp_body = super(ZunClient, self).request(*args, **kwargs)
         if (CONTAINER_SERVICE_MICROVERSION and
-            CONTAINER_SERVICE_MICROVERSION
-                != api_version_utils.LATEST_MICROVERSION):
+            CONTAINER_SERVICE_MICROVERSION !=
+                api_version_utils.LATEST_MICROVERSION):
             api_version_utils.assert_version_header_matches_request(
                 self.api_microversion_header_name,
                 get_container_service_api_version(),
@@ -274,9 +287,11 @@ class ZunClient(rest_client.RestClient):
             self.container_uri(container_id, action='rebuild'), None, **kwargs)
 
     def exec_container(self, container_id, command, **kwargs):
-        return self.post(
+        resp, body = self.post(
             self.container_uri(container_id, action='execute'),
             '{"command": "%s"}' % command, **kwargs)
+        return self.deserialize(
+            resp, body, container_model.ContainerExecEntity)
 
     def logs_container(self, container_id, **kwargs):
         return self.get(
@@ -329,7 +344,7 @@ class ZunClient(rest_client.RestClient):
                 return True
             else:
                 return False
-        utils.wait_for_condition(is_container_in_desired_state, timeout=120)
+        utils.wait_for_condition(is_container_in_desired_state, timeout=240)
 
     def ensure_container_deleted(self, container_id):
         def is_container_deleted():
@@ -339,7 +354,7 @@ class ZunClient(rest_client.RestClient):
                 return False
             else:
                 return True
-        utils.wait_for_condition(is_container_deleted)
+        utils.wait_for_condition(is_container_deleted, timeout=240)
 
     def ensure_action_finished(self, container_id, request_id):
         def is_action_finished():
@@ -388,7 +403,7 @@ class ZunClient(rest_client.RestClient):
                 return True
             else:
                 return False
-        utils.wait_for_condition(is_capsule_in_desired_state, timeout=120)
+        utils.wait_for_condition(is_capsule_in_desired_state, timeout=240)
 
     def list_capsules(self, **kwargs):
         resp, body = self.get(self.capsules_uri(), **kwargs)
@@ -409,7 +424,7 @@ class ZunClient(rest_client.RestClient):
                 return False
             else:
                 return True
-        utils.wait_for_condition(is_capsule_deleted)
+        utils.wait_for_condition(is_capsule_deleted, timeout=240)
 
     def delete_network(self, network_id, params=None):
         return self.delete(self.network_uri(network_id, params=params))
